@@ -1,22 +1,5 @@
-/*
- * music player command (mpc)
- * Copyright 2003-2018 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #ifndef MPC_ARGS_H
 #define MPC_ARGS_H
@@ -33,6 +16,9 @@ struct int_value_change {
 int
 stdinToArgArray(char ***array);
 
+int
+stdinAndPreambleToArgArray(char ***array, char *);
+
 void
 free_pipe_array(unsigned max, char **array);
 
@@ -40,12 +26,22 @@ gcc_pure
 bool
 contains_absolute_path(unsigned argc, char **argv);
 
+gcc_pure
+bool
+contains_absolute_path_from(unsigned argc, char **argv, unsigned from);
+
 void
 strip_trailing_slash(char *s);
 
 int
 get_boolean(const char *arg);
 
+/**
+ * @return true on success
+ */
+bool
+parse_unsigned(const char *s, unsigned *value_r);
+
 /**
  * @return true on success
  */
