@@ -1,32 +1,18 @@
-/*
- * music player command (mpc)
- * Copyright 2003-2018 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "command.h"
 #include "charset.h"
 #include "options.h"
+#include "strcasecmp.h"
 #include "util.h"
 #include "args.h"
 #include "search.h"
 #include "status.h"
+#include "status_format.h"
 #include "tags.h"
 #include "path.h"
+#include "group.h"
 #include "Compiler.h"
 
 #include <mpd/client.h>
@@ -566,6 +552,28 @@ cmd_move(gcc_unused int argc, char **argv, struct mpd_connection *conn)
 	return 0;
 }
 
+int
+cmd_moveplaylist(gcc_unused int argc, char **argv, struct mpd_connection *conn)
+{
+	const char* playlist = argv[0];
+
+	int from;
+	if (!parse_int(argv[1], &from) || from <= 0)
+		DIE("\"%s\" is not a positive integer\n", argv[1]);
+
+	int to;
+	if (!parse_int(argv[2], &to) || to <= 0)
+		DIE("\"%s\" is not a positive integer\n", argv[2]);
+
+	/* users type in 1-based numbers, mpd uses 0-based */
+	--from;
+	--to;
+
+	if (!mpd_run_playlist_move(conn, playlist, from, to))
+		printErrorAndExit(conn);
+	return 0;
+}
+
 int
 cmd_listall(int argc, char **argv, struct mpd_connection *conn)
 {
@@ -580,26 +588,18 @@ cmd_listall(int argc, char **argv, struct mpd_connection *conn)
 		strip_trailing_slash(tmp);
 
 		if (options.custom_format) {
-			bool command_list = false;
-
-#if LIBMPDCLIENT_CHECK_VERSION(2,12,0)
 			/* ask MPD to omit the tags which are not used
 			   by the `--format` to reduce network
 			   transfer for tag values we're not going to
-			   use anyway (requires MPD 0.21 and
-			   libmpdclient 2.12) */
-			if (mpd_connection_cmp_server_version(conn, 0, 21, 0) >= 0) {
-				if (!mpd_command_list_begin(conn, false) ||
-				    !send_tag_types_for_format(conn, options.format))
-					printErrorAndExit(conn);
-				command_list = true;
-			}
-#endif
+			   use anyway */
+			if (!mpd_command_list_begin(conn, false) ||
+			    !send_tag_types_for_format(conn, options.format))
+				printErrorAndExit(conn);
 
 			if (!mpd_send_list_all_meta(conn, tmp))
 				printErrorAndExit(conn);
 
-			if (command_list && !mpd_command_list_end(conn))
+			if (!mpd_command_list_end(conn))
 				printErrorAndExit(conn);
 
 			print_entity_list(conn, MPD_ENTITY_TYPE_SONG, true);
@@ -709,19 +709,14 @@ ls_entity(int argc, char **argv, struct mpd_connection *conn,
 	if (argc > 0)
 		ls = charset_to_utf8(argv[i]);
 
-#if LIBMPDCLIENT_CHECK_VERSION(2,12,0)
 	/* ask MPD to omit the tags which are not used by the
 	   `--format` to reduce network transfer for tag values we're
-	   not going to use anyway (requires MPD 0.21 and libmpdclient
-	   2.12) */
-	if (mpd_connection_cmp_server_version(conn, 0, 21, 0) >= 0) {
-		if (!mpd_command_list_begin(conn, false) ||
-		    !send_tag_types_for_format(conn, options.custom_format ? options.format : NULL) ||
-		    !mpd_command_list_end(conn))
-			printErrorAndExit(conn);
-		my_finishCommand(conn);
-	}
-#endif
+	   not going to use anyway */
+	if (!mpd_command_list_begin(conn, false) ||
+	    !send_tag_types_for_format(conn, options.custom_format ? options.format : NULL) ||
+	    !mpd_command_list_end(conn))
+		printErrorAndExit(conn);
+	my_finishCommand(conn);
 
 	do {
 		if (!mpd_send_list_meta(conn, ls))
@@ -749,19 +744,112 @@ cmd_lsplaylists(int argc, char **argv, struct mpd_connection *conn)
 	return ls_entity(argc, argv, conn, MPD_ENTITY_TYPE_PLAYLIST);
 }
 
+int
+cmd_lsdirs(int argc, char **argv, struct mpd_connection *conn)
+{
+	for (int i = 0; i < argc; i++)
+		strip_trailing_slash(argv[i]);
+
+	return ls_entity(argc, argv, conn, MPD_ENTITY_TYPE_DIRECTORY);
+}
+
+int
+cmd_addplaylist(int argc, char **argv, struct mpd_connection *conn)
+{
+	const char* playlist = argv[0];
+
+	if (contains_absolute_path_from(argc, argv, 1) && !path_prepare(conn))
+		printErrorAndExit(conn);
+
+	if (!mpd_command_list_begin(conn, false))
+		printErrorAndExit(conn);
+
+	for (int i = 1; i < argc; ++i) {
+		strip_trailing_slash(argv[i]);
+
+		const char *path = argv[i];
+		const char *relative_path = to_relative_path(path);
+		if (relative_path != NULL)
+			path = relative_path;
+
+		if (options.verbosity >= V_VERBOSE)
+			printf("adding: %s\n", path);
+		if (!mpd_send_playlist_add(conn, playlist, charset_to_utf8(path))) {
+			printErrorAndExit(conn);
+		}
+	}
+
+	mpd_command_list_end(conn);
+	my_finishCommand(conn);
+
+	return 0;
+}
+
+int
+cmd_delplaylist(int argc, char **argv, struct mpd_connection *conn)
+{
+	const char* playlist = argv[0];
+
+	if (!mpd_command_list_begin(conn, false))
+		printErrorAndExit(conn);
+
+	unsigned position;
+
+	for (int i = 1; i < argc; ++i) {
+
+		if (!parse_unsigned(argv[i], &position)) {
+			DIE("Failed to parse unsigned number: %s\n", argv[i]);
+		}
+
+		if (options.verbosity >= V_VERBOSE)
+			printf("del: %d\n", position);
+
+		/* mpc's song positions are 1-based, but MPD uses
+		   0-based positions */
+		--position;
+
+		if (!mpd_send_playlist_delete(conn, playlist, position)) {
+			printErrorAndExit(conn);
+		}
+	}
+
+	mpd_command_list_end(conn);
+	my_finishCommand(conn);
+
+	return 0;
+}
+
+int
+cmd_renplaylist(int argc, char **argv, struct mpd_connection *conn)
+{
+	(void)argc; // silence warning about unused argument
+	const char* playlist = argv[0];
+	const char* newplaylist = argv[1];
+
+	if (!mpd_run_rename(conn, playlist, newplaylist))
+		printErrorAndExit(conn);
+
+	return 0;
+}
+
+int
+cmd_clearplaylist(int argc, char **argv, struct mpd_connection *conn)
+{
+	(void)argc; // silence warning about unused argument
+	const char* playlist = argv[0];
+
+	if (!mpd_run_playlist_clear(conn, playlist))
+		printErrorAndExit(conn);
+
+	return 0;
+}
+
 int
 cmd_load(int argc, char **argv, struct mpd_connection *conn)
 {
 	const bool range = options.range.start > 0 ||
 		options.range.end < UINT_MAX;
 
-#if !LIBMPDCLIENT_CHECK_VERSION(2,16,0)
-	if (range) {
-		fprintf(stderr, "Loading with range requires libmpdclient 2.16\n");
-		return -1;
-	}
-#endif
-
 	if (!mpd_command_list_begin(conn, false))
 		printErrorAndExit(conn);
 
@@ -769,13 +857,11 @@ cmd_load(int argc, char **argv, struct mpd_connection *conn)
 		printf("loading: %s\n",argv[i]);
 
 		const char *name_utf8 = charset_to_utf8(argv[i]);
-#if LIBMPDCLIENT_CHECK_VERSION(2,16,0)
 		if (range)
 			mpd_send_load_range(conn, name_utf8,
 					    options.range.start,
 					    options.range.end);
 		else
-#endif
 			mpd_send_load(conn, name_utf8);
 	}
 
@@ -814,18 +900,43 @@ cmd_list(int argc, char **argv, struct mpd_connection *conn)
 	--argc;
 	++argv;
 
+	struct mpc_groups groups;
+	mpc_groups_init(&groups);
+	if (!mpc_groups_collect(&groups, &argc, argv))
+		return -1;
+
 	mpd_search_db_tags(conn, type);
 
 	if (argc > 0 && !add_constraints(argc, argv, conn))
 		return -1;
 
+	if (!mpc_groups_send(conn, &groups))
+		printErrorAndExit(conn);
+
 	if (!mpd_search_commit(conn))
 		printErrorAndExit(conn);
 
-	struct mpd_pair *pair;
-	while ((pair = mpd_recv_pair_tag(conn, type)) != NULL) {
-		printf("%s\n", charset_from_utf8(pair->value));
-		mpd_return_pair(conn, pair);
+	if (groups.n_groups > 0) {
+		struct mpd_pair *pair;
+		while ((pair = mpd_recv_pair(conn)) != NULL) {
+			enum mpd_tag_type t = mpd_tag_name_iparse(pair->name);
+			//printf("|| %s\n", pair->name);
+			if (t != MPD_TAG_UNKNOWN) {
+				int i = mpc_groups_find(&groups, t);
+				if (i < 0)
+					i = groups.n_groups;
+				if (i >= 0)
+					printf("%*s%s\n", i * 4, "",
+					       charset_from_utf8(pair->value));
+			}
+			mpd_return_pair(conn, pair);
+		}
+	} else {
+		struct mpd_pair *pair;
+		while ((pair = mpd_recv_pair_tag(conn, type)) != NULL) {
+			printf("%s\n", charset_from_utf8(pair->value));
+			mpd_return_pair(conn, pair);
+		}
 	}
 
 	my_finishCommand(conn);
@@ -944,8 +1055,39 @@ cmd_random(int argc, char **argv, struct mpd_connection *conn)
 int
 cmd_single(int argc, char **argv, struct mpd_connection *conn)
 {
-	return bool_cmd(argc, argv, conn,
-			mpd_status_get_single, mpd_run_single);
+	enum mpd_single_state mode = MPD_SINGLE_UNKNOWN;
+
+	if (argc == 1) {
+		if (strcasecmp(argv[0], "once") == 0)
+			mode = MPD_SINGLE_ONESHOT;
+		else {
+			int mode_i = get_boolean(argv[0]);
+			if (mode_i < 0)
+				return -1;
+			else if (mode_i)
+				mode = MPD_SINGLE_ON;
+			else
+				mode = MPD_SINGLE_OFF;
+		}
+	} else {
+		struct mpd_status *status;
+		status = getStatus(conn);
+		enum mpd_single_state cur = mpd_status_get_single_state(status);
+
+		if (cur == MPD_SINGLE_ONESHOT || cur == MPD_SINGLE_ON)
+			mode = MPD_SINGLE_OFF;
+		else if (cur == MPD_SINGLE_OFF)
+			mode = MPD_SINGLE_ON;
+
+		mpd_status_free(status);
+	}
+
+	if (mode == MPD_SINGLE_UNKNOWN)
+		return -1;
+	else if (!mpd_run_single_state(conn, mode))
+		printErrorAndExit(conn);
+
+	return 1;
 }
 
 int
@@ -991,7 +1133,8 @@ cmd_mixrampdb(int argc, char **argv, struct mpd_connection *conn)
 	} else {
 		struct mpd_status *status = getStatus(conn);
 
-		printf("mixrampdb: %f\n", mpd_status_get_mixrampdb(status));
+		printf("mixrampdb: %f\n",
+		       (double)mpd_status_get_mixrampdb(status));
 
 		mpd_status_free(status);
 	}
@@ -1014,7 +1157,7 @@ cmd_mixrampdelay(int argc, char **argv, struct mpd_connection *conn)
 		struct mpd_status *status = getStatus(conn);
 
 		printf("mixrampdelay: %f\n",
-		       mpd_status_get_mixrampdelay(status));
+		       (double)mpd_status_get_mixrampdelay(status));
 
 		mpd_status_free(status);
 	}
@@ -1090,11 +1233,19 @@ cmd_stats(gcc_unused int argc, gcc_unused char **argv,
 }
 
 int
-cmd_status(gcc_unused int argc, gcc_unused char **argv,
-	   struct mpd_connection *conn)
+cmd_status(int argc, char **argv, struct mpd_connection *conn)
 {
-	if (options.verbosity >= V_DEFAULT)
-		print_status(conn);
+	if (options.verbosity >= V_DEFAULT) {
+		if (argc == 0) {
+			print_status(conn);
+		} else if (argc == 1) {
+			struct mpd_status *status = getStatus(conn);
+			char* current_status = format_status(status, argv[0]);
+			if (current_status) {
+				printf("%s\n", format_status(status, argv[0]));
+			}
+		}
+	}
 	return 0;
 }
 
@@ -1122,3 +1273,56 @@ cmd_replaygain(int argc, char **argv, struct mpd_connection *connection)
 
 	return 0;
 }
+
+int
+cmd_partitionlist(gcc_unused int argc, gcc_unused char **argv, struct mpd_connection *conn)
+{
+	if (!mpd_send_listpartitions(conn)) {
+		printErrorAndExit(conn);
+	}
+
+	struct mpd_partition *part;
+	while ((part = mpd_recv_partition(conn)) != NULL) {
+		printf("%s\n", mpd_partition_get_name(part));
+		mpd_partition_free(part);
+	}
+
+	my_finishCommand(conn);
+	return 0;
+}
+
+int cmd_partitionmake(int argc, char **argv, struct mpd_connection *conn)
+{
+	if (!mpd_command_list_begin(conn, false)) {
+		printErrorAndExit(conn);
+	}
+
+	for (int i = 0; i < argc; ++i) {
+		if (!mpd_send_newpartition(conn, argv[i])) {
+			printErrorAndExit(conn);
+		}
+	}
+
+	if (!mpd_command_list_end(conn) || !mpd_response_finish(conn)) {
+		printErrorAndExit(conn);
+	}
+	return 0;
+}
+
+int
+cmd_partitiondelete(int argc, char **argv, struct mpd_connection *conn) {
+	if (!mpd_command_list_begin(conn, false)) {
+		printErrorAndExit(conn);
+	}
+
+	for (int i = 0; i < argc; ++i) {
+		if (!mpd_send_delete_partition(conn, argv[i])) {
+			printErrorAndExit(conn);
+		}
+	}
+
+	if (!mpd_command_list_end(conn) || !mpd_response_finish(conn)) {
+		printErrorAndExit(conn);
+	}
+	return 0;
+}
