@@ -1,25 +1,9 @@
-/*
- * music player command (mpc)
- * Copyright 2003-2018 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "search.h"
 #include "options.h"
+#include "strcasecmp.h"
 #include "tags.h"
 #include "util.h"
 #include "charset.h"
@@ -28,11 +12,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <strings.h>
 
 enum {
 	SEARCH_TAG_ANY = MPD_TAG_COUNT + 1,
-	SEARCH_TAG_URI = MPD_TAG_COUNT + 2,
+	SEARCH_TAG_URI,
 	SEARCH_TAG_BASE,
 };
 
@@ -45,13 +28,13 @@ static enum mpd_tag_type
 get_search_type(const char *name)
 {
 	if (strcasecmp(name, "any") == 0)
-		return SEARCH_TAG_ANY;
+		return (enum mpd_tag_type)SEARCH_TAG_ANY;
 
 	if (strcasecmp(name, "filename") == 0)
-		return SEARCH_TAG_URI;
+		return (enum mpd_tag_type)SEARCH_TAG_URI;
 
 	if (strcasecmp(name, "base") == 0)
-		return SEARCH_TAG_BASE;
+		return (enum mpd_tag_type)SEARCH_TAG_BASE;
 
 	enum mpd_tag_type type = mpd_tag_name_iparse(name);
 	if (type != MPD_TAG_UNKNOWN)
@@ -77,16 +60,14 @@ static int
 get_constraints(int argc, char **argv, struct constraint **constraints)
 {
 	int numconstraints = 0;
-	int type;
-	int i;
 
 	assert(argc > 0 && argc % 2 == 0);
 
 	*constraints = malloc(sizeof(**constraints) * argc / 2);
 
-	for (i = 0; i < argc; i += 2) {
-		type = get_search_type(argv[i]);
-		if (type < 0) {
+	for (int i = 0; i < argc; i += 2) {
+		enum mpd_tag_type type = get_search_type(argv[i]);
+		if (type == MPD_TAG_UNKNOWN) {
 			free(*constraints);
 			return -1;
 		}
@@ -121,14 +102,9 @@ add_constraint(struct mpd_connection *conn,
 bool
 add_constraints(int argc, char ** argv, struct mpd_connection *conn)
 {
-#if LIBMPDCLIENT_CHECK_VERSION(2,16,0)
 	if (argc == 1 && argv[0][0] == '(') {
-		if (mpd_connection_cmp_server_version(conn, 0, 21, 0) < 0)
-			fprintf(stderr, "warning: MPD 0.21 required for search expressions\n");
-
 		return mpd_search_add_expression(conn, argv[0]);
 	}
-#endif
 
 	struct constraint *constraints;
 
@@ -150,20 +126,12 @@ add_constraints(int argc, char ** argv, struct mpd_connection *conn)
 static int
 do_search(int argc, char ** argv, struct mpd_connection *conn, bool exact)
 {
-	bool command_list = false;
-
-#if LIBMPDCLIENT_CHECK_VERSION(2,12,0)
 	/* ask MPD to omit the tags which are not used by the
 	   `--format` to reduce network transfer for tag values we're
-	   not going to use anyway (requires MPD 0.21 and libmpdclient
-	   2.12) */
-	if (mpd_connection_cmp_server_version(conn, 0, 21, 0) >= 0) {
-		if (!mpd_command_list_begin(conn, false) ||
-		    !send_tag_types_for_format(conn, options.custom_format ? options.format : NULL))
-			printErrorAndExit(conn);
-		command_list = true;
-	}
-#endif
+	   not going to use anyway */
+	if (!mpd_command_list_begin(conn, false) ||
+	    !send_tag_types_for_format(conn, options.custom_format ? options.format : NULL))
+		printErrorAndExit(conn);
 
 	mpd_search_db_songs(conn, exact);
 	if (!add_constraints(argc, argv, conn))
@@ -172,7 +140,7 @@ do_search(int argc, char ** argv, struct mpd_connection *conn, bool exact)
 	if (!mpd_search_commit(conn))
 		printErrorAndExit(conn);
 
-	if (command_list && !mpd_command_list_end(conn))
+	if (!mpd_command_list_end(conn))
 		printErrorAndExit(conn);
 
 	print_entity_list(conn, MPD_ENTITY_TYPE_SONG, options.custom_format);
