@@ -1,26 +1,10 @@
-/*
- * music player command (mpc)
- * Copyright 2003-2018 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "status.h"
-#include "charset.h"
+#include "status_format.h"
 #include "util.h"
+#include "charset.h"
 #include "mpc.h"
 
 #include <mpd/client.h>
@@ -29,20 +13,6 @@
 #include <stdio.h>
 #include <string.h>
 
-static unsigned
-elapsed_percent(const struct mpd_status *status)
-{
-	unsigned total = mpd_status_get_total_time(status);
-	if (total == 0)
-		return 0;
-
-	unsigned elapsed = mpd_status_get_elapsed_time(status);
-	if (elapsed >= total)
-		return 100;
-
-	return (elapsed * 100) / total;
-}
-
 void
 print_status(struct mpd_connection *conn)
 {
@@ -105,9 +75,12 @@ print_status(struct mpd_connection *conn)
 	else printf("off   ");
 
 	printf("single: ");
-	if (mpd_status_get_single(status))
+	if (mpd_status_get_single_state(status) == MPD_SINGLE_ON)
 		printf("on    ");
-	else printf("off   ");
+	else if (mpd_status_get_single_state(status) == MPD_SINGLE_ONESHOT)
+		printf("once  ");
+	else if (mpd_status_get_single_state(status) == MPD_SINGLE_OFF)
+		printf("off   ");
 
 	printf("consume: ");
 	if (mpd_status_get_consume(status))
