@@ -1,22 +1,5 @@
-/*
- * music player command (mpc)
- * Copyright 2003-2018 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "util.h"
 #include "song_format.h"
@@ -107,11 +90,13 @@ print_entity_list(struct mpd_connection *c, enum mpd_entity_type filter_type,
 
 		case MPD_ENTITY_TYPE_SONG:
 			song = mpd_entity_get_song(entity);
-			if (pretty) {
-				pretty_print_song(song);
-				puts("");
-			} else
-				printf("%s\n", charset_from_utf8(mpd_song_get_uri(song)));
+			if (!options.with_prio || mpd_song_get_prio(song) > 0) {
+				if (pretty) {
+					pretty_print_song(song);
+					puts("");
+				} else
+					printf("%s\n", charset_from_utf8(mpd_song_get_uri(song)));
+			}
 			break;
 
 		case MPD_ENTITY_TYPE_PLAYLIST:
